import sys
import argparse


def setup_args_env(arg_list):
    for arg in arg_list:
        sys.argv.append(arg)


def get_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('--input-dir', help='Input directory', default='.')
    parser.add_argument('--output-dir', help='Output directory', default='output')
    parser.add_argument('--overwrite', help='Whether to overwrite output', type=int, default=1)
    return parser



import os

class TotalSegmentator:

    def __init__(self, nifti_path, output_dir):
        self.nifti_path = nifti_path
        self.output_dir = output_dir
        self.fast = False
        self.statistics = True
        self.radiomics = False

    def execute(self):
        fast = ''
        if self.fast:
            fast = '--fast'
        statistics = ''
        if self.statistics:
            statistics = '--statistics'
        radiomics = ''
        if self.radiomics:
            radiomics = '--radiomics'
        cmd = 'TotalSegmentator {} {} {} -i {} -o {}'.format(
            statistics, 
            radiomics,
            fast,
            self.nifti_path,
            self.output_dir,
        )
        print(f'Running command: {cmd}')
        os.system(cmd)


class RoiSliceExtractor:
    """ Extracts all slices in the source image that intersect
    the given ROI
     """
    def __init__(self, image_path, roi_path):
        self.image_path = image_path
        self.roi_path = roi_path

    def execute(self):
        pass


if __name__ == '__main__':
    def main():
        pass
    main()



import argparse
from barbell2.castor import loaders
from barbell2.castor import builders


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('castor_export', help='Castor export Excel file')
    parser.add_argument('dica_export', help='DHBA or DPCA export Excel data file')
    parser.add_argument('dica_dd', help='DHBA or DPCA data dictionary')
    parser.add_argument('output_dir', help='Output directory to store generated CSV files')
    args = parser.parse_args()
    cast_loader = loaders.CastorExportFileLoader(args.castor_export)
    cast_data = cast_loader.load()
    dica_loader = loaders.DicaExportFileLoader(args.dica_export, args.dica_dd)
    dica_data = dica_loader.load()
    builder = builders.CastorRecordBuilder(cast_data, dica_data)
    builder.execute()
    builder.save(args.output_dir)


def bla():
    cast_loader = loaders.CastorExportFileLoader('/Users/Ralph/data/surfdrive/documents/hpb/castor/data/exports/castoredc/ESPRESSO_v2.0_DHBA_excel_export_20220926102336.xlsx')
    cast_data = cast_loader.load()
    dica_loader = loaders.DicaExportFileLoader(
        '/Users/Ralph/data/surfdrive/documents/hpb/castor/data/exports/dhba/dhba_2022.academisch-ziekenhuis-maastricht.xlsx',
        '/Users/Ralph/data/surfdrive/documents/hpb/castor/data/data_dicitionaries/dhba/dhba-2022_1.1.0_datadictionary_20211001_122752.xlsx',
    )
    dica_data = dica_loader.load()
    builder = builders.CastorRecordBuilder(cast_data, dica_data)
    builder.execute()
    builder.save('/Users/Ralph/Desktop/buildcastorimport')


if __name__ == '__main__':
    bla()



import os
import shutil
import argparse

from random import shuffle


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--input_dir', help='Input directory', default='.')
    parser.add_argument('--output_dir', help='Output directory', default='output')
    parser.add_argument(
        '--nr_folds', help='Number of folds in cross-validation (0 means no CV)', type=int, default=0)
    parser.add_argument('--test_size', help='Percentage to use for testing', type=float, default=0.25)
    parser.add_argument('--validation_size', help='Percentage to use for validation/tuning', type=float, default=0)
    parser.add_argument('--shuffle', help='Whether to shuffle files before splitting', type=int, default=1)
    args = parser.parse_args()
    training_size = 1.0 - args.validation_size - args.test_size
    assert training_size >= 0.25, f'Training size < 25% ({training_size})'
    if args.nr_folds > 0:
        assert args.nr_folds >= 3, f'Number of cross-validation folds < 3 ({args.nr_folds})'
    os.makedirs(args.output_dir)

    # TODO: Shuffle taking DICOM and TAG file pairs into account!!!
    # Load DICOM/TAG file pairs together
    # all_files = []
    # for f in os.listdir(args.input_dir):
    #     f_path = os.path.join(args.input_dir, f)
    #     all_files.append(f_path)
    # if args.shuffle == 1:
    #     shuffle(all_files)
    # nr_test = int(args.test_size * len(all_files))
    # nr_validation = int(args.validation_size * len(all_files))
    # nr_train = len(all_files) - nr_validation - nr_test
    # if nr_train > 0:
    #     os.makedirs(os.path.join(args.output_dir, 'training'))
    #     for i in range(nr_train):
    #         f_path = all_files.pop(0)
    #         f_name = os.path.split(f_path)[1]
    #         f_path_new = os.path.join(args.output_dir, 'training', f_name)
    #         shutil.copy(f_path, f_path_new)
    # if nr_validation > 0:
    #     os.makedirs(os.path.join(args.output_dir, 'validation'))
    #     for i in range(nr_validation):
    #         f_path = all_files.pop(0)
    #         f_name = os.path.split(f_path)[1]
    #         f_path_new = os.path.join(args.output_dir, 'validation', f_name)
    #         shutil.copy(f_path, f_path_new)
    # if nr_test > 0:
    #     os.makedirs(os.path.join(args.output_dir, 'testing'))
    #     for i in range(nr_test):
    #         f_path = all_files.pop(0)
    #         f_name = os.path.split(f_path)[1]
    #         f_path_new = os.path.join(args.output_dir, 'testing', f_name)
    #         shutil.copy(f_path, f_path_new)


if __name__ == '__main__':
    from barbell2 import common
    common.setup_args_env([
        '--input_dir=/Users/Ralph/SURF Drive/projects/hpb/bodycomposition/data/l3tag_cohorts/SURG-PANC',
        '--output_dir=/Users/Ralph/Desktop/output',
        '--test_size=0.2',
        '--validation_size=0.1',
    ])
    main()



import os
import time
import math
import datetime
import struct
import binascii
import numpy as np


class Logger(object):

    def __init__(self, prefix='log', to_dir='.', timestamp=True):
        self.f = None
        self.timestamp = timestamp
        os.makedirs(to_dir, exist_ok=True)
        now = datetime.datetime.now()
        if not prefix.endswith('_'):
            prefix = prefix + '_'
        file_name = '{}{}.txt'.format(prefix, now.strftime('%Y%m%d%H%M%S'))
        self.f = open(os.path.join(to_dir, file_name), 'w')

    def print(self, message):
        now = datetime.datetime.now()
        if self.timestamp:
            message = '[' + now.strftime('%Y-%m-%d %H:%M:%S.%f') + '] ' + str(message)
        else:
            message = str(message)
        print(message)
        if self.f:
            self.f.write(message + '\n')

    def close(self):
        self.f.close()

    def __del__(self):
        self.close()


def current_time_millis():
    return int(round(time.time() * 1000))


def current_time_secs():
    return int(round(current_time_millis() / 1000.0))


def elapsed_millis(start_time_millis):
    return current_time_millis() - start_time_millis


def elapsed_secs(start_time_secs):
    return current_time_secs() - start_time_secs


def duration(seconds):
    h = int(math.floor(seconds/3600.0))
    remainder = seconds - h * 3600
    m = int(math.floor(remainder/60.0))
    remainder = remainder - m * 60
    s = int(math.floor(remainder))
    return '{} hours, {} minutes, {} seconds'.format(h, m, s)


def is_dicom_file(file_path_or_obj):
    file_obj = file_path_or_obj
    if isinstance(file_obj, str):
        if not os.path.isfile(file_obj):
            return False
        if file_obj.startswith('._'):
            return False
        file_obj = open(file_obj, "rb")
    try:
        result = file_obj.read(132).decode('ASCII')[-4:] == 'DICM'
        file_obj.seek(0)
        return result
    except UnicodeDecodeError:
        return False


def is_tag_file(file_path):
    return file_path.endswith('.tag') and not file_path.startswith('._')


def get_tag_file_for_dicom(dcm_file, ext='.tag'):
    tag_file = os.path.splitext(dcm_file)[0] + ext
    if not os.path.isfile(tag_file):
        tag_file = dcm_file + ext
        if not os.path.isfile(tag_file):
            print(f'Could not find TAG file for DICOM file {dcm_file}')
            return None
        return tag_file
    return tag_file


def is_numpy_file(file_path):
    return file_path.endswith('.npy') and not file_path.startswith('._')


def get_numpy_file_for_dicom(dcm_file):
    return get_tag_file_for_dicom(dcm_file, ext='.npy')


def get_pixels(p, normalize=False):
    pixels = p.pixel_array
    if not normalize:
        return pixels
    if normalize is True:
        return p.RescaleSlope * pixels + p.RescaleIntercept
    if isinstance(normalize, int):
        return (pixels + np.min(pixels)) / (np.max(pixels) - np.min(pixels)) * normalize
    if isinstance(normalize, list):
        return (pixels + np.min(pixels)) / (np.max(pixels) - np.min(pixels)) * normalize[1] + normalize[0]
    return pixels


def get_tag_pixels(tag_file_path):
    f = open(tag_file_path, 'rb')
    f.seek(0)
    byte = f.read(1)
    # Make sure to check the byte-value in Python 3!!
    while byte != b'':
        byte_hex = binascii.hexlify(byte)
        if byte_hex == b'0c':
            break
        byte = f.read(1)
    values = []
    f.read(1)
    while byte != b'':
        v = struct.unpack('b', byte)
        values.append(v)
        byte = f.read(1)
    values = np.asarray(values)
    values = values.astype(np.uint16)
    return values


def get_alberta_color_map():
    color_map = []
    for i in range(256):
        if i == 1:  # muscle
            color_map.append([255, 0, 0])
        elif i == 2:  # inter-muscular adipose tissue
            color_map.append([0, 255, 0])
        elif i == 5:  # visceral adipose tissue
            color_map.append([255, 255, 0])
        elif i == 7:  # subcutaneous adipose tissue
            color_map.append([0, 255, 255])
        elif i == 12:  # unknown
            color_map.append([0, 0, 255])
        else:
            color_map.append([0, 0, 0])
    return color_map


def apply_window(pix, window):
    result = (pix - window[1] + 0.5 * window[0])/window[0]
    result[result < 0] = 0
    result[result > 1] = 1
    return result


def apply_color_map(pixels, color_map):
    pixels_new = np.zeros((*pixels.shape, 3), dtype=np.uint8)
    np.take(color_map, pixels, axis=0, out=pixels_new)
    return pixels_new


def create_fake_dicom(pixels, dcm_obj):
    pixels_new = apply_color_map(pixels, get_alberta_color_map())
    dcm_obj.PhotometricInterpretation = 'RGB'
    dcm_obj.SamplesPerPixel = 3
    dcm_obj.BitsAllocated = 8
    dcm_obj.BitsStored = 8
    dcm_obj.HighBit = 7
    dcm_obj.add_new(0x00280006, 'US', 0)
    dcm_obj.is_little_endian = True
    dcm_obj.fix_meta_info()
    dcm_obj.PixelData = pixels_new.tobytes()
    dcm_obj.SOPInstanceUID = '{}.9999'.format(dcm_obj.SOPInstanceUID)
    return dcm_obj



import os
import shutil
import pydicom
import dcmstack
import npy2png

from dcmstack import dcmmeta


class Dicom2Nifti:

    def __init__(self, dcm_file_dir, nii_file_path):
        self.dcm_file_dir = dcm_file_dir
        items = os.path.split(nii_file_path)
        self.nii_file_name = items[1]
        if self.nii_file_name.endswith('.gz'):
            self.nii_file_name = self.nii_file_name[:-7]
        elif self.nii_file_name.endswith('.nii'):
            self.nii_file_name = self.nii_file_name[:-4]
        else:
            pass
        self.nii_file_dir = items[0]
        self.cmd = f'dcm2niix -m y -z y -f {self.nii_file_name} -o {self.nii_file_dir} {self.dcm_file_dir}'

    def execute(self):
        print(f'Executing command {self.cmd}')
        os.system(self.cmd)


class Dicom2NiftiWithHeaderInfo:

    def __init__(self, dcm_file_dir, nii_file):
        self.dcm_file_dir = dcm_file_dir
        self.nii_file = nii_file

    def execute(self):
        files = [os.path.join(self.dcm_file_dir, f) for f in os.listdir(self.dcm_file_dir)]
        print('searching for slice 100...')
        f_100 = None
        for f in files:
            p = pydicom.dcmread(f)
            if int(p.InstanceNumber) == 100:
                print('found!')
                f_100 = f
                break
        shutil.copyfile(f_100, '/Users/Ralph/Desktop/f_100.dcm')
        # https://dcmstack.readthedocs.io/en/v0.6.1/Python_Tutorial.html#looking-up-meta-data
        stacks = dcmstack.parse_and_stack(files, group_by='SeriesInstanceUID')
        stack = list(stacks.values())[0]
        nii = stack.to_nifti(embed_meta=True)
        nr_slices = nii.shape[2]
        print(f'nr. slices: {nr_slices}')
        nii.to_filename(self.nii_file)
        # nii_meta = dcmmeta.NiftiWrapper.from_filename('file.nii.gz')
        # for i in range(nr_slices):
        #     instance_nr = nii_meta.get_meta('InstanceNumber', index=(0, 0, i))
        #     if instance_nr == 100:
        #         image = nii_meta.nii_img.dataobj[:,:,i]
        #         print(f'found image! {image.shape}')
        #         n2p = npy2png.Numpy2Png(image)
        #         n2p.set_output_dir('/Users/Ralph/Desktop')
        #         n2p.set_png_file_name('f_100.dcm.png')
        #         n2p.execute()
        #         break


if __name__ == '__main__':
    def main():
        # d2n = Dicom2Nifti(
        #     '/Users/Ralph/data/scalpel/raw/tlodewick-ct-noise-1/AL_100%/101816478/2-Abdomen',
        #     '/Users/Ralph/data/scalpel/processed/tlodewick-ct-noise-1-out-1/my_nifti.nii.gz',
        # )
        # d2n.execute()
        # print(os.listdir('/Users/Ralph/data/scalpel/processed/tlodewick-ct-noise-1-out-1'))
        d2n = Dicom2NiftiWithHeaderInfo(
            '/Users/Ralph/Desktop/itk/ct-abdomen',
            '/Users/Ralph/Desktop/itk/out/ct-abdomen.nii.gz',
        )
        d2n.execute()
    main()



import numpy as np
import nrrd


class Numpy2Nrrd:

    def __init__(self, npy_array_or_file_path, output_file_path):
        self.npy_array_or_file_path = npy_array_or_file_path
        self.output_file_path = output_file_path
        self.index_order = 'C'

    def execute(self):
        if isinstance(self.npy_array_or_file_path, str):
            npy_array = np.load(self.npy_array_or_file_path)
        else:
            npy_array = self.npy_array_or_file_path
        nrrd.write(self.output_file_path, npy_array)


def main():
    from barbell2.imaging.tag2npy import Tag2Numpy
    t2n = Tag2Numpy('data/file.tag', shape=(512, 512))
    npy_array = t2n.execute()
    print(np.unique(npy_array))
    np.save('data/file.npy', npy_array)
    n2n = Numpy2Nrrd(npy_array, 'data/file.nrrd')
    n2n.execute()


if __name__ == '__main__':
    main()



import os
import pytest

from barbell2.createh5 import CreateH5

base_dir = '{}/data/surfdrive/projects/20210203_autosegl3'.format(os.environ['HOME'])


def clean_up():
    if os.path.isfile('train.h5'):
        os.remove('train.h5')
    if os.path.isfile('test.h5'):
        os.remove('test.h5')
    if os.path.isfile('predict.h5'):
        os.remove('predict.h5')


# def test_train_test():
#     """ Both DICOM and TAG files should be collected and the data split up in a training and test set. """
#     clean_up()
#     d = os.path.join(base_dir, 'dicom_and_tag')
#     args = {
#         'data_dir': d,
#         'rows': 512, 'columns': 512,
#         'type': 'train',
#         'test_size': 0.2,
#         'log_dir': '.',
#     }
#     x = CreateH5(args)
#     x.execute()
#     assert os.path.isfile('train.h5')
#     assert os.path.isfile('test.h5')
#     clean_up()


def test_train():
    clean_up()
    # d = os.path.join(base_dir, 'dicom_and_tag')
    d = '/Volumes/USB_SECURE1/data/radiomics/projects/deepseg/data/mega/processed'
    args = {
        'data_dir': d,
        'output_dir': os.path.join(os.environ['HOME'], 'Desktop/h5'),
        'rows': 512, 'columns': 512,
        'type': 'train',
        'test_size': 0.0,
        'log_dir': '.',
    }
    x = CreateH5(args)
    x.execute()
    assert os.path.isfile(os.path.join(args['output_dir'], 'train.h5'))
    clean_up()


# def test_predict():
#     clean_up()
#     d = os.path.join(base_dir, 'dicom')
#     args = {
#         'data_dir': d,
#         'rows': 512, 'columns': 512,
#         'type': 'predict',
#         'test_size': 0.0,
#         'log_dir': '.',
#     }
#     x = CreateH5(args)
#     x.execute()
#     assert os.path.isfile('predict.h5')
#     clean_up()
